#!/usr/bin/perl -w
# ======================================================================                                                                                                  
# CiteDiff - Citation Diff Tool
# ====================================================================== 
# The script matchs two concordance csv file and write in a file 
# the result of comparaison. 
# The concordance csv files for input are obtained by CiteExtract.pl 
# ======================================================================  
# Copyright (C) 2012 ESIEE Paris, Université Paris-Est
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Ayoung KIM <pherlnim-at-gmail.com> 
# ======================================================================            

use strict;
use warnings;
# ======================================================================
use utf8;
use Readonly;
use Data::Dumper;
use File::Slurp::Unicode;
use YAML::Tiny;
use Text::CSV;
use String::Approx 'amatch';
# ======================================================================

# ======================================================================                 
# Constants 
# ======================================================================
#
Readonly my $RES_PTC 		=> "~~";	# Partially correct
Readonly my $RES_CRT 		=> "==";	# Correct
Readonly my $RES_MIS 		=> "+-";	# Missing
Readonly my $RES_FAP 		=> "-+";	# False positive
Readonly my $RES_MSM 		=> "<>";	# Mismatch

Readonly my $SYR_NWL 		=> "↵";		# New line
Readonly my $SYR_TAB 		=> "↹";		# Tab
Readonly my $SYR_SPC 		=> " ";		# Space

Readonly my $SYM_NWL1 	=> "\r";	# New line
Readonly my $SYM_NWL2 	=> "\n";	# New line
Readonly my $SYM_TAB 		=> "\t";	# Tab
Readonly my $SYM_SPC 		=> " ";		# Space

Readonly my $CONTEXT_WIDTH	=> 30;		# Size of context for printing matches
Readonly my $ATTR_CONTEXT_WIDTH	=> 15;		# Size of context for printing attributes matches

Readonly my $CSV_HEADER 	=> "No,Start,End,Tag,Context,Tp,Context,Tag,No,Start,End\n";

Readonly my $CONFIG_YAML =>  "config.yml";	# Configuration file name in YAML format
Readonly my $DIR_EXTRACTED => "extracted/";	# Directory name which contatins concordance files

#DO NOT MODIFY THIS SECTION : LOADED BY CONFIGURATION FILE
Readonly my $CONFIG => YAML::Tiny->read($CONFIG_YAML) or die 'Can\'t find config file : $cfg_filename';	# Configuration loaded
Readonly my $TAGS_AVAILABLE => $CONFIG->[0]->{tag} or die "No tag definition";	# Availabe tag list 
Readonly my $ATTR_SPR => $CONFIG->[0]->{params}->{attribute} or die "No attribute separator definition"; 	# Attribute separator 
Readonly my $SUB_ATTR_SPR => $CONFIG->[0]->{params}->{subattribute} or die "No sub-attribute separator definition";	# Sub-attribute separators

# ======================================================================                 
# Functions 
# ======================================================================
sub escapestring($){
	my $str = shift;
	$str =~ s/$SYM_SPC/$SYR_SPC/g;	# Replace spaces
	$str =~ s/$SYM_TAB/$SYR_TAB/g;	# Replace tabs
	$str =~ s/$SYM_NWL1$SYM_NWL2/$SYR_NWL/g;	# Replace new lines (\r\n\n)
	$str =~ s/$SYM_NWL1/$SYR_NWL/g;	# Replace new lines (\r\n)
	$str =~ s/$SYM_NWL2/$SYR_NWL/g;	# Replace new lines (\n)
	return $str;
}

sub match_citations($$$){
	my ($keycsv_filename, $rescsv_filename, $mode) = @_;
	
	open my $key_fh, "<:encoding(utf8)", $keycsv_filename or die "Cannot find ".$keycsv_filename;
	open my $res_fh, "<:encoding(utf8)", $rescsv_filename or die "Cannot find ".$keycsv_filename;
	unless ($mode == 1 or $mode ==0) {
		die "3rd parameter : 0 (matching level 1) or 1 (matching level 2)";
	}
	my $csv = Text::CSV->new({
		binary=>1,
		allow_loose_quotes=>1,
		escape_char=>"\\"
		}) or die "Cannot use CSV: ".Text::CSV->error_diag();

	#Loading lines in csv files	
	my (@key_rows, @res_rows,$output);
	$output = '';
	while(my $key_row = $csv->getline($key_fh)) {
		push @key_rows, $key_row;
	}	
	$csv->eof or $csv->error_diag();
	close $key_fh;
	while(my $res_row = $csv->getline($res_fh)) {
		push @res_rows, $res_row;
	}	
	$csv->eof or $csv->error_diag();
	close $res_fh;
	
	my $key_size = @key_rows;
	my $res_size = @res_rows;
	my $empty_line = ['','','','','','','',''];
	my $key_idx = 0;
	my $res_idx = 0;
	
	#loop until we arrive the end of an array
	while($key_idx<$key_size and $res_idx<$res_size) {
		#if the current index position in array is empty, redo loop with next index
		unless (defined($key_rows[$key_idx])) { $key_idx++; next; };
		unless (defined($res_rows[$res_idx])) { $res_idx++; next; };
		
		#get contexts
		my $key_context = $key_rows[$key_idx]->[5];
		my $res_context = $res_rows[$res_idx]->[5];
		my $key_left = $key_rows[$key_idx]->[4];
		my $res_left = $res_rows[$res_idx]->[4];
		my $key_right = $key_rows[$key_idx]->[6];
		my $res_right = $res_rows[$res_idx]->[6];
		
		#clues for decide partitial matching
		my $key_start = $key_rows[$key_idx]->[1];
		my $key_end = $key_rows[$key_idx]->[2];
		my $res_start = $res_rows[$res_idx]->[1];
		
		#delete tags in context string
		$key_context =~ s/<.*?>//g;
		$res_context =~ s/<.*?>//g;
		
		#replace new line and tab symbols to a space
		$key_context=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
		$res_context=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
		$key_left=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
		$res_left=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
		$key_right=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
		$res_right=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;

		#2 citations match 'near' perfectely
		if (amatch($key_context, $res_context) and amatch($key_left, $res_left) and amatch($key_right, $res_right)) {
			$output = $output.build_matching_output($key_rows[$key_idx], $res_rows[$res_idx], $RES_CRT);

			#if we choose attribute-level matching, excute attribute matching
			if($mode==1) { $output = $output.(match_attributes($key_rows[$key_idx], $res_rows[$res_idx])); }
			$key_idx++;
			$res_idx++;
	
		#2 citations match partially (maybe res_context is a substring of key_context and it starts between key's start and end) 
		} elsif((index($key_context, $res_context) != -1) or (index($res_context, $key_context) != -1) ) {
			$output = $output.build_matching_output($key_rows[$key_idx], $res_rows[$res_idx], $RES_PTC);

			if($mode==1) { $output = $output.(match_attributes($key_rows[$key_idx], $res_rows[$res_idx])); }
			$key_idx++;
			$res_idx++;
		
		#and if 2 citations doesn't match!
		} else {
			#we will see if there exist a matching line with key_line within remains of res_rows
			my $tmp_idx = $res_idx+1;
			while ($tmp_idx<$res_size) {
				#get contexts of candidate
				my $tmp_context = $res_rows[$tmp_idx]->[5];
				my $tmp_left = $res_rows[$tmp_idx]->[4];
				my $tmp_right = $res_rows[$tmp_idx]->[6];

				#use start index for decide partitial matching
				my $tmp_start = $res_rows[$tmp_idx]->[1];

				#delete tag and replace newline/tab symbols to a space
				$tmp_context =~ s/<.*?>//g;
				$tmp_context =~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
				$tmp_left =~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
				$tmp_right =~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
					
				#if a matching line found in another index of response (tmp_idx)
				if (amatch($key_context, $tmp_context) and amatch($key_left, $tmp_left) and amatch($key_right, $tmp_right) ) {

					#false positives : response lines between res_idx and tmp_idx
					while($res_idx < $tmp_idx) {
						$output = $output.build_matching_output($empty_line, $res_rows[$res_idx], $RES_FAP);
						if($mode==1) { $output = $output.(match_attributes($empty_line, $res_rows[$res_idx])); }
						##log message for Eleni!
						#print "Warning : Missed citation? : ".$res_rows[$res_idx]->[5]."\n";
						$res_idx++;
					}
					
					#write matched line 
					$output = $output.build_matching_output($key_rows[$key_idx], $res_rows[$res_idx], $RES_CRT);
					if($mode==1) { $output = $output.(match_attributes($key_rows[$key_idx], $res_rows[$res_idx])); }
					$key_idx++;
					$res_idx++;

					last;
				#else if a partitial matching line found in another index of response (tmp_idx)
				}  elsif((index($key_context, $tmp_context) != -1) or (index($tmp_context, $key_context) != -1)) {

					#false positives : response lines between res_idx and tmp_idx
					while($res_idx < $tmp_idx) {
						$output = $output.build_matching_output($empty_line, $res_rows[$res_idx], $RES_FAP);
						if($mode==1) { $output = $output.(match_attributes($empty_line, $res_rows[$res_idx])); }
						##log message for Eleni!
						#print "Warning : Missed citation? : ".$res_rows[$res_idx]->[5]."\n";
						$res_idx++;
					}
					
					#write partitially matched line
					$output = $output.build_matching_output($key_rows[$key_idx], $res_rows[$res_idx], $RES_PTC);
					if($mode==1) { $output = $output.(match_attributes($key_rows[$key_idx], $res_rows[$res_idx])); }
					$key_idx++;
					$res_idx++;
					last;		
				} else {
					$tmp_idx++;
					next;
				}
			}
			#if there are no matching item, current key line will be a missing.
			if ($tmp_idx>=$res_size){
				$output = $output.build_matching_output($key_rows[$key_idx], $empty_line, $RES_MIS);
				if($mode==1) { $output = $output.(match_attributes($key_rows[$key_idx], $empty_line));	}
				$key_idx++;
			}
		}
	}
	
	#if there exist remains in key lines - they will be missings
	if($key_idx < $key_size) {
		while($key_idx<$key_size) {
			$output = $output.build_matching_output($key_rows[$key_idx], $empty_line, $RES_MIS);
			#print build_matching_output($key_rows[$key_idx], $empty_line, $RES_MIS);;
				if($mode==1) { $output = $output.(match_attributes($key_rows[$key_idx], $empty_line));	}
			$key_idx++;
		}
	}
	
	#if there exist remains in res lines - they will be false positives
	if($res_idx < $res_size) {
		while($res_idx<$res_size) {
			$output = $output.build_matching_output($empty_line, $res_rows[$res_idx], $RES_FAP);
			#print build_matching_output($empty_line, $res_rows[$res_idx], $RES_FAP);
			if($mode==1) { $output = $output.(match_attributes($empty_line, $res_rows[$res_idx])); }
			$res_idx++;
		}
	}
	
	close $key_fh;
	close $res_fh;
	return $output;
}

#build a string which contains some information of two given lines of csv with their match type  
sub build_matching_output($$$) {
	my ($current_key, $current_res, $match_type) = @_;
	my $output = '"'.$current_key->[0].'","'.$current_key->[1].'","'.$current_key->[2].'","'.$current_key->[3].'","'.$current_key->[4].$current_key->[5].$current_key->[6].'",';
	$output = $output."\"$match_type\",";
	$output = $output.'"'.$current_res->[4].$current_res->[5].$current_res->[6].'","'.$current_res->[3].'","'.$current_res->[0].'","'.$current_res->[1].'","'.$current_res->[2].'"';
	$output = $output."\n";

	return $output;
}

sub match_attributes($$) {
	my ($key_line, $res_line) = @_;
	my $output = '';
	
	#get attributes;
	my @key_attrs = split(quotemeta($ATTR_SPR), $key_line->[7]);
	my @res_attrs = split(quotemeta($ATTR_SPR), $res_line->[7]);
	
	my $dump_attr;
	if($key_line->[3] ne '') {
		$dump_attr = Dumper $CONFIG->[0]->{$key_line->[3]};
	} else {
		$dump_attr = Dumper $CONFIG->[0]->{$res_line->[3]};
	}
	my $_no=1;	
	foreach my $key_item  (@key_attrs) {
		if(index($key_item,'=')==-1) {next;}
		my $res_item='';
		my @key_item_array = split('=', $key_item, 2);
		my $key_label = $key_item_array[0];	
		my $key_value = $key_item_array[1];
		unless(defined($key_label)) {next;}
		##error log for Eleni! 
		#unless(grep(/$key_label/, $dump_attr)) { 
			#print "Warning : At ".$key_line->[0]." unknown attribute name : '$key_label'\n";
			#next;
		#}
		#check the number of attribute items which have same attribute name
		my $count_candidate=0;
		while ($res_line->[7] =~ /$key_label=/g) { $count_candidate++ }

		#if key item contain sub-attributes.
		my @sub_key_attrs;
		if(defined($key_value) and index($key_value , $SUB_ATTR_SPR) != -1) { 
			@sub_key_attrs = split(quotemeta($SUB_ATTR_SPR), $key_value); 
			$key_item_array[1] = $key_value = $sub_key_attrs[0];
			splice (@sub_key_attrs, 0, 1);
		}
		
		#find a response attribute which contains key attribute, and delete it from the list of attribute candidates
		for(my $i=0; $i<scalar(@res_attrs); $i++) { 
			if(index($res_attrs[$i],'=')== -1) {next;}
			unless(defined($res_attrs[$i])) { next; }
			#candidate = attribute items with a same label
			my $tmp_label = (split('=', $res_attrs[$i], 2))[0];
			if(defined($tmp_label) and ($key_label eq $tmp_label)) {
				#if we have just a candidate, get it
				if($count_candidate==1) {
					$res_item = $res_attrs[$i];
					splice (@res_attrs, $i, 1);
					last;
				#else we have more than one candidate, we will check them all until we find a matching. 
				} else {
					for(my $j=0; $j<scalar(@res_attrs); $j++) {
						unless(defined($res_attrs[$j])) { next; }
						#get attribute's value to compare with key's value
						my @tmp_item_array = split('=', $res_attrs[$j], 2);
						my $tmp_label = $tmp_item_array[0];
						my $tmp_value = $tmp_item_array[1];
						unless(defined($tmp_label)) { next; }
						if($key_label eq $tmp_label) {
							if(index($tmp_value , $SUB_ATTR_SPR) != -1) { $tmp_value =  (split(quotemeta($SUB_ATTR_SPR), $tmp_value))[0]; }
							#if we find a matching(lenient), get it
							if((index($key_value, $tmp_value)!=-1) or (amatch($key_value, $tmp_value))) {
								$res_item = $res_attrs[$j];
								splice (@res_attrs, $j, 1);
								last;
							}  
						}
					}
					#if we arrive here, there were no matching between the candidates..that will be missing.
					last;
				}
			}
		} 
		my @res_item_array = split('=', $res_item, 2);
		my $res_label = $res_item_array[0];
		my $res_value = $res_item_array[1];
		#if res item contain sub-attributes.
		my @sub_res_attrs;
		my $has_sub_attr_flag = 0;
		if(defined($res_value) and index($res_value , $SUB_ATTR_SPR) != -1) { 
			@sub_res_attrs = split(quotemeta($SUB_ATTR_SPR), $res_value); 
			$res_item_array[1]=$res_value = $sub_res_attrs[0];
			splice (@sub_res_attrs, 0, 1);
			$has_sub_attr_flag = 1;
		}

		#print matched attributes
		my $k_no = $key_line->[0].".$_no";
		my $r_no = $res_line->[0].".$_no";
		$output = $output.build_matching_attr_output($key_line, $res_line, $k_no, $r_no, \@key_item_array, \@res_item_array);	
		#if current key attribute contains sub-attributes.
		if($has_sub_attr_flag== 1) {
			my $__no=0;
			foreach my $sub_key_item (@sub_key_attrs) {
				my $sub_res_item='' ;
				if(index($sub_key_item, '=') == -1) { next; }
				else {
					my @sub_key_item_array = split('=',$sub_key_item, 2);
					my $sub_key_label = $sub_key_item_array[0];
					my $sub_key_value = $sub_key_item_array[1];
					unless(grep(/$sub_key_label/,$dump_attr)) { next;}
					for(my $i=0; $i<scalar(@sub_res_attrs); $i++) {
						unless(defined($sub_res_attrs[$i])) { next; }
						my @sub_tmp_item_array = split('=', $sub_res_attrs[$i], 2);
						my $sub_tmp_label = $sub_tmp_item_array[0];
						my $sub_tmp_value = $sub_tmp_item_array[1];
						#find a response sub-attribute which has a same label with key sub-attribute
						if($sub_key_label eq $sub_tmp_label) {
							$sub_res_item = $sub_res_attrs[$i];
							splice (@sub_res_attrs, $i, 1);
							last;
						}
					}	
					#if sub_res_item is undefined at this point, it will be missing
					my @sub_res_item_array = split('=', $sub_res_item, 2);
					$k_no = $key_line->[0].".$_no.$__no";
					$r_no = $res_line->[0].".$_no.$__no";
					$output = $output.build_matching_attr_output($key_line, $res_line, $k_no, $r_no, \@sub_key_item_array, \@sub_res_item_array);
					$__no++;
				}
			} 
			#false-positive ..if there are remains in sub_res_attrs..
			if(scalar(@sub_res_attrs) != 0) {
				foreach my $sub_res_item (@sub_res_attrs) {
					if ($sub_res_item eq '') { next; }
					if(index($sub_res_item, '=') == -1) { next; }
					my @sub_key_item_array;
					my @sub_res_item_array =  split('=', $sub_res_item,2);
					$k_no = '';
					$r_no = $res_line->[0].".$_no.$__no";
					$output = $output.build_matching_attr_output($key_line, $res_line, $k_no, $r_no, \@sub_key_item_array, \@sub_res_item_array);
					$__no++;	
				}
			}
			$_no++; 
		} 
	}
	#false-positive ..if there are remains in res_attrs..
	if(scalar(@res_attrs) != 0) {
		foreach my $res_item (@res_attrs) {
			if ($res_item eq '') { next; }	
			if(index($res_item, '=') == -1) { next; }	
			my @key_item_array;
			my @res_item_array= split('=', $res_item,2);
			my $res_label= $res_item_array[0];
			my $res_value = $res_item_array[1];
			unless(grep(/$res_label/, $dump_attr)) { next;}

			#if res item contain sub-attributes.
			my @sub_res_attrs;
			my $has_sub_attr_flag = 0;
			if(index($res_value , $SUB_ATTR_SPR) != -1) { 
				@sub_res_attrs = split(quotemeta($SUB_ATTR_SPR), $res_value); 
				$res_item_array[1] = $res_value = $sub_res_attrs[0];
				splice (@sub_res_attrs, 0, 1);
				$has_sub_attr_flag = 1;
			}

			#print attributes
			my $k_no = '';
			my $r_no = $res_line->[0].".$_no";
			$output = $output.build_matching_attr_output($key_line, $res_line, $k_no, $r_no, \@key_item_array, \@res_item_array);	
		
			#print sub attributes
			if($has_sub_attr_flag== 1) {
				my $__no=0;
				foreach my $sub_res_item (@sub_res_attrs) {
					if(index($sub_res_item, '=') == -1) { next; }
					else {
						my @sub_key_item_array;
						my @sub_res_item_array = split('=',$sub_res_item,2);
						my $sub_res_label = $sub_res_item_array[0];
						#todo.. have to find another way for check accepted attributes. 
						unless(grep(/$sub_res_label/,$dump_attr)) { next;}
						$r_no = $res_line->[0].".$_no.$__no";
						$output = $output.build_matching_attr_output($key_line, $res_line, $k_no, $r_no, \@sub_key_item_array, \@sub_res_item_array);
						$__no++;
					}
				}
			} 
			$_no++;
		}
	}
	return $output;
}

#build a string which contains some information of two given attribute lines with their match type 
sub build_matching_attr_output($$$$$$) {
	my ($key_line, $res_line, $k_no, $r_no, $key_item_array, $res_item_array) = @_;

	my $type;
	if(!defined($key_item_array->[1])) {
		$type = $RES_FAP;
		$k_no = '';
	} elsif (!defined($res_item_array->[1])) {
		$type = $RES_MIS;
		$r_no = '';
	} elsif ($key_item_array->[1] eq $res_item_array->[1]) {
		$type = $RES_CRT;
	} elsif ((index($key_item_array->[1], $res_item_array->[1]) != -1) or (amatch($key_item_array->[1], $res_item_array->[1]))) {
		$type = $RES_PTC;
		##log message for Eleni!
		#my $str1 = $res_item_array->[1];
		#my $str2 = $key_item_array->[1];
		#$str1 =~ s/[, :.]//g ;
		#$str2 =~ s/[, :.]//g ;
		#if($str1 ne $str2) {
			#my $line_no = $key_line->[0];
			#print "Warning : At $line_no need to verify attribute '".$key_item_array->[0]."' :  '".$key_item_array->[1]."'? -> '".$res_item_array->[1]."'\n";
		#}
	} else {
		$type = $RES_MSM ;
	}
		
	#ignore tag
	my $key_context = $key_line->[5];
	my $res_context = $res_line->[5];
	$key_context =~ s/<.*?>//g;
	$res_context =~ s/<.*?>//g;

	#ignore new line and tab symbols
	$key_context=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
	$res_context=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g;
	if(defined($key_item_array->[1])) { $key_item_array->[1]=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g; }
	if(defined($res_item_array->[1])) { $res_item_array->[1]=~ s/($SYR_NWL|$SYR_TAB)/$SYR_SPC/g; }
			
	my ($k_label, $k_start, $k_end, $k_left, $k_right, $k_context);
	my ($r_label, $r_start, $r_end, $r_left, $r_right, $r_context);
	
	#todo:bad start index if the attribute value is different with original text.. ex)'et_al' 
	if($type ne $RES_FAP) {
		if($key_item_array->[0] eq 'et_al') {
			$k_label = $key_item_array->[0];
		} else {
			my $k_inner_index = index($key_context, $key_item_array->[1]) ;
			my $k_full_context = $key_line->[4].$key_context.$key_line->[6];		
			$k_label = $key_item_array->[0];
			$k_start =  $k_inner_index+ $key_line->[1];
			$k_end = $k_start+(length $key_item_array->[1]);					
			$k_left = substr($k_full_context, $CONTEXT_WIDTH+$k_inner_index-$ATTR_CONTEXT_WIDTH, $ATTR_CONTEXT_WIDTH);
			$k_right = substr($k_full_context, $CONTEXT_WIDTH+$k_inner_index+(length $key_item_array->[1]), $ATTR_CONTEXT_WIDTH);

			$k_context = "\<$k_label\>".$key_item_array->[1]."\</$k_label\>";
		}
	}
	if($type ne $RES_MIS) {
		if($res_item_array->[0] eq 'et_al') {
			$r_label = $res_item_array->[0];
		} else {
			my $r_inner_index = index($res_context, $res_item_array->[1]) ;
			my $r_full_context = $res_line->[4].$res_context.$res_line->[6];
			$r_label = $res_item_array->[0];
			$r_start =  $r_inner_index+ $res_line->[1];
			$r_end = $r_start+(length $res_item_array->[1]);
			$r_left = substr($r_full_context, $CONTEXT_WIDTH+$r_inner_index-$ATTR_CONTEXT_WIDTH, $ATTR_CONTEXT_WIDTH);
			$r_right = substr($r_full_context, $CONTEXT_WIDTH+$r_inner_index+(length $res_item_array->[1]), $ATTR_CONTEXT_WIDTH);
			$r_context = "\<$r_label\>".$res_item_array->[1]."\</$r_label\>";
		}
	}
	
	unless(defined($k_start)) {$k_start='';}
	unless(defined($k_end)) {$k_end='';}
	unless(defined($k_label)) {$k_label='';}
	unless(defined($k_left)) {$k_left='';}
	unless(defined($k_right)) {$k_right='';}
	unless(defined($k_context)) {$k_context='';}
	unless(defined($r_start)) {$r_start='';}
	unless(defined($r_end)) {$r_end='';}
	unless(defined($r_label)) {$r_label='';}
	unless(defined($r_left)) {$r_left='';}
	unless(defined($r_right)) {$r_right='';}
	unless(defined($r_context)) {$r_context='';}
	
	#escape quotes for writing in csv format
	$k_left =~ s/\"/\\\"/g;
	$k_right =~ s/\"/\\\"/g;
	$k_context =~ s/\"/\\\"/g;
	$r_left =~ s/\"/\\\"/g;
	$r_right =~ s/\"/\\\"/g;
	$r_context =~ s/\"/\\\"/g;
	
	my $output = '"'.$k_no.'","'.$k_start.'","'.$k_end.'","'.$k_label.'","'.$k_left.$k_context.$k_right.'",';
	$output = $output."\"$type\",";
	$output = $output.'"'.$r_left.$r_context.$r_right.'","'.$r_label.'","'.$r_no.'","'.$r_start.'","'.$r_end.'"';
	$output = $output."\n";

	return $output;
}


# ======================================================================                 
# Main program  
# ====================================================================== 
#
binmode(STDOUT, "utf8");

my $num_args = $#ARGV + 1;
if ($num_args != 3) {
  print "\nUsage: CiteDiff.pl key_corpus response_corpus output_filename\n";
  exit;
}

my $result_v1 = match_citations($ARGV[0], $ARGV[1], 1);
write_file($ARGV[2],$result_v1);


