#!/usr/bin/perl -w
# ======================================================================                                                                                                  
# CiteEval - Citation Evaluate Tool
# ====================================================================== 
# The script calculates and print recall, precision, and fscore value 
# from the result csv file of CiteDiff.pl.
# ======================================================================  
# Copyright (C) 2012 ESIEE Paris, Université Paris-Est
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Ayoung KIM <pherlnim-at-gmail.com> 
# ======================================================================            

use strict;
use warnings;
# ======================================================================
use utf8;
use Readonly;
use Data::Dumper;
use File::Slurp::Unicode;
use YAML::Tiny;
use Text::CSV;
use String::Approx 'amatch';
# ======================================================================

# ======================================================================                 
# Constants 
# ======================================================================
#
Readonly my $RES_PTC 		=> "~~";	# Partially correct
Readonly my $RES_CRT 		=> "==";	# Correct
Readonly my $RES_MIS 		=> "+-";	# Missing
Readonly my $RES_FAP 		=> "-+";	# False positive
Readonly my $RES_MSM 		=> "<>";	# Mismatch

Readonly my $CONFIG_YAML =>  "config.yml";	# Configuration file name in YAML format

#DO NOT MODIFY THIS SECTION : LOADED BY CONFIGURATION FILE
Readonly my $CONFIG => YAML::Tiny->read($CONFIG_YAML) or die 'Can\'t find config file : $cfg_filename';	# Configuration loaded
Readonly my $TAGS_AVAILABLE => $CONFIG->[0]->{tag} or die "No tag definition";	# Availabe tag list 
Readonly my $ATTR_SPR => $CONFIG->[0]->{params}->{attribute} or die "No attribute separator definition"; 	# Attribute separator 
Readonly my $SUB_ATTR_SPR => $CONFIG->[0]->{params}->{subattribute} or die "No sub-attribute separator definition";	# Sub-attribute separators

# ======================================================================                 
# Functions 
# ======================================================================
sub evaluate($$) {
	my ($resultcsv_filename, $tag_list) = @_;
	my ($tp, $fp, $fn,$tpa);	
	$tp=$fp=$fn=$tpa=0;

	open my $result_fh, "<:encoding(utf8)", $resultcsv_filename or die "Cannot find ".$resultcsv_filename;
	my $csv = Text::CSV->new({
		binary=>1,
		allow_loose_quotes=>1,
		escape_char=>"\\"
		}) or die "Cannot use CSV: ".Text::CSV->error_diag();
	my $dump_tags;
	if(ref($tag_list) eq 'ARRAY') {$dump_tags = Dumper @$tag_list;} 
	else { $dump_tags = $tag_list; }
	my $cnt=1;
	while(my $row = $csv->getline($result_fh)) {
		if (ref($tag_list) eq 'ARRAY') { 
			my $tag =$row->[3];
			if ($tag eq '') {$tag = $row->[7];}
			unless(grep(/$tag/, $dump_tags)) { next;} 
		}
		if($row->[5] eq $RES_CRT) { $tp++; }
		elsif($row->[5] eq $RES_FAP) { $fp++; }
		elsif($row->[5] eq $RES_MIS) { $fn++; } 
		elsif($row->[5] eq $RES_PTC) { $tpa++;}
	}
	close $result_fh;

	my ($s_precision, $s_recall, $s_fscore, $l_precision, $l_recall, $l_fscore, $a_precision, $a_recall, $a_fscore);
	if ($tp == 0) { $s_precision = 0; $s_recall=0; $s_fscore=0;}
	else {
		$s_precision = 100 * $tp / ($tp+$fp);
		$s_recall = 100 * $tp / ($tp+$fn);
		$s_fscore = ($s_precision+$s_recall)/2;
	}
	if ($tp+$tpa == 0) { $l_precision =0; $l_recall=0; $l_fscore=0; }
	else {
		$l_precision = 100 * ($tp+$tpa) / (($tp+$tpa)+$fp);
		$l_recall = 100 * ($tp+$tpa) / (($tp+$tpa)+$fn);
		$l_fscore = ($l_precision+$l_recall)/2;
	}
	if($l_fscore==0) { $a_precision=0; $a_recall=0; $a_fscore=0;}
	else {
		$a_precision = ($s_precision+$l_precision)/2;
		$a_recall = ($s_recall+$l_recall)/2;
		$a_fscore = ($s_fscore+$l_fscore)/2;
	}
	print "*************************************************************\n";
if(ref($tag_list) eq 'ARRAY') {
	print "\tScore Calculator for CiteDiff - Tag level\n";
}else {
	print "\tScore Calculator for CiteDiff - Attribute level\n";
}
	print "*************************************************************\n";
	print "#Correct=$tp, #Part_Correct=$tpa, #False-Positiv=$fp, #Missing=$fn\n";
	print "*************************************************************\n";
	print "\t\tStrict\t\tLenient\t\tAvr\n";
	printf ("Precision\t%.2f\t\t%.2f\t\t%.2f\n",$s_precision,$l_precision,$a_precision);
	printf ("Recall\t\t%.2f\t\t%.2f\t\t%.2f\n",$s_recall,$l_recall,$a_recall);
	printf ("F-score\t\t%.2f\t\t%.2f\t\t%.2f\n",$s_fscore,$l_fscore,$a_fscore);
	print "*************************************************************\n";
	
}

# ======================================================================                 
# Main program  
# ====================================================================== 
#
print "=======RESULT for $ARGV[0]========\n";
evaluate($ARGV[0], ['citation', 'reference', 'patentref']);
evaluate($ARGV[0], 'all');
print "\n";
