#!/usr/bin/perl -w
# ======================================================================                                                                                                  
# CiteDiff - Citation Extract Tool
# ====================================================================== 
# The script extract citations in a corpus annotated, and save their   
#  informations in a concordance csv file.
# The result file will be created in 'extracted' folder. 
# ======================================================================  
# Copyright (C) 2012 ESIEE Paris, Université Paris-Est
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Ayoung KIM <pherlnim-at-gmail.com> 
# ======================================================================            

use strict;
use warnings;
# ======================================================================
use utf8;
use Readonly;
use Data::Dumper;
use File::Slurp::Unicode;
use YAML::Tiny;
# ======================================================================

# ======================================================================                 
# Constants 
# ======================================================================
#

Readonly my $SYR_NWL 		=> "↵";		# New line
Readonly my $SYR_TAB 		=> "↹";		# Tab
Readonly my $SYR_SPC 		=> " ";		# Space

Readonly my $SYM_NWL1 	=> "\r";	# New line
Readonly my $SYM_NWL2 	=> "\n";	# New line
Readonly my $SYM_TAB 		=> "\t";	# Tab
Readonly my $SYM_SPC 		=> " ";		# Space

Readonly my $CONTEXT_WIDTH	=> 30;		# Size of context for printing matches

Readonly my $CONFIG_YAML =>  "config.yml";	# Configuration file name in YAML format
Readonly my $DIR_EXTRACTED => "extracted/";	# Directory name which contatins concordance files

#DO NOT MODIFY THIS SECTION : LOADED BY CONFIGURATION FILE
Readonly my $CONFIG => YAML::Tiny->read($CONFIG_YAML) or die 'Can\'t find config file : $cfg_filename';	# Configuration loaded
Readonly my $TAGS_AVAILABLE => $CONFIG->[0]->{tag} or die "No tag definition";	# Availabe tag list 

# ======================================================================                 
# Functions 
# ======================================================================
sub escapestring($){
	my $str = shift;
	$str =~ s/$SYM_SPC/$SYR_SPC/g;	# Replace spaces
	$str =~ s/$SYM_TAB/$SYR_TAB/g;	# Replace tabs
	$str =~ s/$SYM_NWL1$SYM_NWL2/$SYR_NWL/g;	# Replace new lines (\r\n\n)
	$str =~ s/$SYM_NWL1/$SYR_NWL/g;	# Replace new lines (\r\n)
	$str =~ s/$SYM_NWL2/$SYR_NWL/g;	# Replace new lines (\n)
	return $str;
}

#create CSV format concordance with a corpus which contains annotated citation
#parameter : A corpus annotated
#return : A string which contains the information of extracted citations
sub create_concordance($){
	my ($corpus) = @_;
	my ($no, $start, $end, $tag,$left,$context,$right,$attribute);
	my ($open_tag , $close_tag, $corpus_without_annotation,$output, $close_tag_without_info);
	
	#get lists of opening tags and closing tags
	#for example, if we have 'citation' and 'patentref' under 'tag' field in the configuration file,
	#we will get $open_tag=['<citation>', '<patentref'], $close_tag=['</citation ', '</patentref ']
	$open_tag = '';
	$close_tag = '';
	for my $item (@{$TAGS_AVAILABLE }) {
		$open_tag = $open_tag."\<$item\>\|";
		$close_tag = $close_tag."\<\/$item \|"."\<$item \|"."\/$item \|";
		$close_tag_without_info = $close_tag_without_info."\<\/$item>\|";
	}
	$open_tag= substr($open_tag,0,-1);
	$close_tag= substr($close_tag,0,-1);
	$close_tag_without_info= substr($close_tag_without_info,0,-1);

	#create the document without annotations to get easier left/right context information 
	$corpus_without_annotation = $corpus;
	$corpus_without_annotation =~ s/$open_tag//g;
	$corpus_without_annotation =~ s/($close_tag)(.*?)>//g;
	pos $corpus_without_annotation = 0;
	
	##corpus cleaning
	##if we find a string in format '<string>' in corpus without annotation,
	##we will delete them in both of corpus annotated and corpus non-annotated
	#while($corpus_without_annotation =~ s/<\w(.*?)\w>//g) {
		#print "Deleted in cleaning phase: ".$&."\n";
		#$corpus =~ s/$&//g;
	#} 
	#pos $corpus = 0;
	#pos $corpus_without_annotation = 0;

	$no = 1;
	#when we find one of opening tags in the corpus annotated
	while ($corpus =~ /$open_tag/g) {

		#get the tag found
		my $open_tag_found  = $&;
		$tag = substr($&,1,(length $open_tag_found)-2);
		
		#get the start offset 
		$start = pos $corpus;
		
		#find a closing tag after the start position
		$corpus =~ /$close_tag|$close_tag_without_info/g;
		
		#get the end offset 
		my $close_tag_found = $&;
		$end = (pos $corpus)-(length $close_tag_found);
		
		#get the citation text
		$context = substr($corpus, $start, $end-$start);

		if(substr($close_tag_found,-1,1) eq '>') {
			$attribute = "";
		} else { 
			#get the attributes's information (text within '</TAG ' and '>') 
			$corpus =~ /\>/g; 
			$attribute = substr($corpus, $end+(length $open_tag_found)+1, (pos $corpus)-($end+(length $close_tag_found)+1));
			#$attribute = substr($corpus, $end+(length $close_tag_found), (pos $corpus)-($end+(length $close_tag_found)));
		}
		#escape the characters special to use as a pattern for matching
		my $context_quoted = quotemeta($context);
		
		#find the position of citation in the corpus non-annotated 
		$corpus_without_annotation =~ /$context_quoted/g;
		my $offset = pos $corpus_without_annotation;
		unless(defined($offset)) {print $context."\n";}
		#if we can't find the citation in the corpus non-annotated, 
		#retry with 10 first characters of citation
		#TODO : when we had a missing citation with it's similer content, the offset could be incorrect 
		unless (defined($offset)) {
			my $sub = substr($context, 0, 10);
			$context_quoted = quotemeta($sub);
			$corpus_without_annotation =~ /$context_quoted/g;
			$offset = pos $corpus_without_annotation;
		}
	
		#get left context
		$left = substr($corpus_without_annotation, $offset-((length $context)+$CONTEXT_WIDTH), $CONTEXT_WIDTH);
		#get right context
		$right = substr($corpus_without_annotation, $offset, $CONTEXT_WIDTH);

		#escape quotes( "-> \" ) for writing in csv format
		$left =~ s/\"/\\\"/g;
		$right =~ s/\"/\\\"/g;
		$context =~ s/\"/\\\"/g;
		$attribute=~ s/\"/\\\"/g;

		#write in output
		$output = $output."\"$no\",\"$start\",\"$end\",\"$tag\",";
		$output = $output."\"$left\",\"\<$tag\>$context\<\/$tag\>\",\"$right\",\"$attribute\"\n";
		#increase line number
		$no++;
	}
	
	return $output;
}

# ======================================================================                 
# Main program  
# ====================================================================== 
#
binmode(STDOUT, "utf8");

#read command-line parameters
my $num_args = $#ARGV + 1;
if ($num_args != 1) {
  print "\nUsage: CiteExtract.pl corpus_annotated\n";
  exit;
}

#read corpus and convert some characters specials (ex:new line character)
my $corpus = read_file($ARGV[0]) or die 'Can\'t find : $!';
$corpus = escapestring($corpus);
$ARGV[0]= (split("/",$ARGV[0]))[-1];
$ARGV[0] =~ s/\//_/g;
#save the result of extration
my $output_filename = "concord_".$ARGV[0].".csv";
write_file($DIR_EXTRACTED .$output_filename,create_concordance($corpus)) or die "Cannot write".$DIR_EXTRACTED .$output_filename;

